@@ -1,5 +1,5 @@
 from pyeoskit import eosapi, wallet
-eosapi.set_node('https://api.eosn.com')
+eosapi.set_node('https://api.eosn.io')
 
 #import your account private key here
 wallet.import_key('mywallet', '5K463ynhZoCDDa4RDcr63cUwWLTnKqmdcoTKTHBjqoKfv4u5V7p')
