@@ -88,11 +88,11 @@ type ABI struct {
 	Variants         []VariantDef   `json:"variants"`
 }
 
-func (t *ABI) PackAbiType(abiType string, args []byte) ([]byte, error) {
+func (t *ABI) PackAbiType(abiType string, args string) ([]byte, error) {
 	enc := NewEncoder(1024)
 
 	m := make(map[string]JsonValue)
-	err := json.Unmarshal(args, &m)
+	err := json.Unmarshal([]byte(args), &m)
 	if err != nil {
 		return nil, newError(err)
 	}
@@ -119,21 +119,6 @@ func (t *ABI) UnpackAbiType(abiName string, packedValue []byte) ([]byte, error)
 	return bs, nil
 }
 
-func (t *ABI) PackAbiStructByName(structName string, args string) ([]byte, error) {
-	enc := NewEncoder(1024)
-	m := make(map[string]JsonValue)
-	err := json.Unmarshal([]byte(args), &m)
-	if err != nil {
-		return nil, newError(err)
-	}
-
-	err = t.PackAbiStruct(enc, structName, m)
-	if err != nil {
-		return nil, newError(err)
-	}
-	return enc.Bytes(), nil
-}
-
 func (t *ABI) ParseAbiStringValue(enc *Encoder, typ string, v string) error {
 	switch typ {
 	case "bool":
@@ -316,9 +301,9 @@ func (t *ABI) ParseAbiStringValue(enc *Encoder, typ string, v string) error {
 		if err != nil {
 			return newError(err)
 		}
-		n := tt.UnixNano()
+		n := tt.UnixMicro()
 		enc.PackInt64(n)
-	case "time_point_sec", "block_timestamp_type":
+	case "time_point_sec":
 		v, ok := StripString(v)
 		if !ok {
 			return newErrorf("invalid time_point_sec value: %s", v)
@@ -329,6 +314,8 @@ func (t *ABI) ParseAbiStringValue(enc *Encoder, typ string, v string) error {
 		}
 		n := tt.Unix()
 		enc.PackUint32(uint32(n))
+	case "block_timestamp_type":
+		return newErrorf("packing block_timestamp_type type does not supported")
 	case "name":
 		v, ok := StripString(v)
 		if !ok {
@@ -593,14 +580,16 @@ func (t *ABI) unpackAbiStructField(dec *Decoder, typ string) (interface{}, error
 			return nil, newError(err)
 		}
 		//convert seconds to iso8601
-		return time.Unix(0, int64(v)).Format("2006-01-02T15:04:05"), nil
-	case "time_point_sec", "block_timestamp_type":
+		return time.UnixMicro(int64(v)).Format("2006-01-02T15:04:05"), nil
+	case "time_point_sec":
 		v, err := dec.ReadUint32()
 		if err != nil {
 			return nil, newError(err)
 		}
 		//convert seconds to iso8601
 		return time.Unix(int64(v), 0).Format("2006-01-02T15:04:05"), nil
+	case "block_timestamp_type":
+		return nil, newErrorf("block_timestamp_type is not supported in unpacking")
 	case "name":
 		v, err := dec.ReadUint64()
 		if err != nil {
